//  pcm_sink_test.cpp: test case for pcm_sink

//  Copyright Takeshi Mouri 2006.
//  Use, modification, and distribution are subject to the
//  Boost Software License, Version 1.0. (See accompanying file
//  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#include <hamigaki/audio/pcm_device.hpp>
#include <hamigaki/audio/sine_wave.hpp>
#include <hamigaki/audio/stereo.hpp>
#include <boost/iostreams/device/array.hpp>
#include <boost/iostreams/copy.hpp>
#include <boost/iostreams/restrict.hpp>
#include <boost/test/unit_test.hpp>
#include <algorithm>
#include <cmath>
#include <iterator>
#include <vector>

namespace audio = hamigaki::audio;
namespace io = boost::iostreams;
namespace ut = boost::unit_test;

double power(double x, double y)
{
    if (y >= 0.0)
        return std::pow(x, y);
    else
        return std::pow(1.0/x, -y);
}

// calculate the frequency from specified MIDI note number
double calc_frequency(unsigned short note)
{
    return 440.0 * power(2.0, (note-69) / 12.0);
}

template<int Bits>
void pcm_sink_test_aux(unsigned rate, unsigned channels)
{
    audio::pcm_format fmt;
    fmt.rate = rate;
    fmt.bits = Bits;
    fmt.channels = channels;

    if (channels == 1)
    {
        io::copy(
            io::restrict(
                audio::sine_wave_source(rate, Bits, calc_frequency(60)),
                0, rate*((Bits+7)/8)
            ),
            audio::pcm_sink(fmt, rate/5*((Bits+7)/8)*channels)
        );
    }
    else
    {
        io::copy(
            io::restrict(
                audio::stereo(
                    audio::sine_wave_source(rate, Bits, calc_frequency(60))
                ), 0, rate*((Bits+7)/8)*2
            ),
            audio::pcm_sink(fmt, rate/5*((Bits+7)/8)*channels)
        );
    }
}

void pcm_sink_test()
{
    const unsigned rates[] = { 11025, 22050, 44100 };

    for (std::size_t i = 0; i < sizeof(rates)/sizeof(rates[0]); ++i)
    {
        for (unsigned channels = 1; channels <= 2; ++channels)
        {
            pcm_sink_test_aux<8>(rates[i], channels);
            pcm_sink_test_aux<16>(rates[i], channels);
        }
    }
}

ut::test_suite* init_unit_test_suite(int, char* [])
{
    ut::test_suite* test = BOOST_TEST_SUITE("pcm_sink test");
    test->add(BOOST_TEST_CASE(&pcm_sink_test));
    return test;
}
